// Copyright Vespa.ai. Licensed under the terms of the Apache 2.0 license. See LICENSE in the project root.

#pragma once

#include "multisearch.h"
#include <vespa/vespalib/util/priority_queue.h>
#include <vespa/searchlib/fef/matchdata.h>
#include <vespa/searchlib/fef/termfieldmatchdataarray.h>
#include <vespa/searchlib/attribute/posting_iterator_pack.h>

namespace search::fef { class TermFieldMatchData; }

namespace search::queryeval {

/**
 * Search iterator for a sparse dot product, based on a set of child
 * search iterators.
 *
 * This class is a base class for a set of different instantiations of
 * DotProductSearchImpl, defined in the .cpp-file.
 */
class DotProductSearch : public SearchIterator
{
protected:
    DotProductSearch() {}

public:
    static constexpr bool filter_search = false;
    static constexpr bool require_btree_iterators = true;
    static constexpr bool supports_hash_filter = false;

    // TODO: use MultiSearch::Children to pass ownership
    static SearchIterator::UP create(const std::vector<SearchIterator*> &children,
                                     search::fef::TermFieldMatchData &tmd,
                                     bool field_is_filter,
                                     const std::vector<fef::TermFieldMatchData*> &childMatch,
                                     const std::vector<int32_t> &weights,
                                     fef::MatchData::UP md);

    static SearchIterator::UP create(search::fef::TermFieldMatchData &tmd,
                                     bool field_is_filter,
                                     const std::vector<int32_t> &weights,
                                     std::vector<DocidWithWeightIterator> &&iterators);
};

}

